#!/bin/bash
# tpl_functions.sh
# Author : Cédric Villemain
# Functions handling slonik and bash templates

#################################################
# End tpl slonik functions
#
# Standard ones
#
function tpl_slonik_include_preamble {
  local PREAMBLE="$1" ; shift
  cat << TPL
#--
# <preamble> file
#--
include <$PREAMBLE>
TPL
  return 0
}

function tpl_slonik_success_error {
  local MSG_SUCCESS="$1"
  local MSG_ERROR="$2"
  shift 2

  cat << TPL
on error {
  echo '$MSG_ERROR';
  exit 1;
}
on success {
  echo '$MSG_SUCCESS';
}
TPL
  return 0
}

function tpl_slonik_sync_id {
  local ID="$1" ; shift
  echo "sync (ID = ${ID});"
  return 0
}

function tpl_slonik_wait_for_event {
  local ORIGIN="$1"
  local CONFIRMED="$2"
  local WAIT="$3"
  shift 3
  cat << TPL
  wait for event (ORIGIN = ${ORIGIN},
				  CONFIRMED = ${CONFIRMED},
				  WAIT ON = ${WAIT});
TPL
  return 0
}

#
# with TRY
#
function tpl_slonik_store_node {
  local ID="$1"
  local EVENT_NODE="$2"
  local DB="$3"
  local HOST="$4"
  local CLUSTER="$5"
  local MSG="$6"
  shift 6

  cat << TPL
#--
# Create the node $ID (slave)
# Usually followed by
# the path that tell the nodes how to connect to
# each other and how they should listen for events.
#--
echo '$MSG';
try{
  store node (ID = $ID,
			  EVENT NODE = $EVENT_NODE,
  comment = 'DB $DB - Host $HOST - Cluster $CLUSTER');
}
TPL
  return 0
}

function tpl_slonik_drop_node {
  local ID="$1"
  local EVENT_NODE="$2"
  local MSG="$3"
  shift 3

  cat << TPL
echo '$MSG' ;
try {
  drop node ( ID = $ID,
			  EVENT NODE = $EVENT_NODE);
}
TPL
  return 0
}

function tpl_slonik_subscribe_set {
  local ID="$1"
  local PROVIDER="$2"
  local RECEIVER="$3"
  local MSG="$4"
  shift 4

  cat << TPL
# ----
# $MSG
# ----
try{
  echo '$MSG';
  subscribe set ( ID = $ID,
				  PROVIDER = $PROVIDER,
				  RECEIVER = $RECEIVER,
				  FORWARD = YES );
}
TPL
  return 0
}

function tpl_slonik_merge_set {
  local ID="$1"
  local ADD_ID="$2"
  local ORIGIN="$3"
  shift 3

  cat << TPL
#--
# Merging the sets
#--
echo 'Merge Set $ID - $ADD_ID -> $ID: ';
try{
  merge set (ID = $ID,
			 ADD ID = $ADD_ID,
			 ORIGIN = $ORIGIN );
}
TPL
  return 0
}

function tpl_slonik_init_cluster {
  # use global SLONY_PWD if defined and SLONY_USER
  local ID="$1"
  local CLUSTER="$2"
  local NODEID="$3"
  local DB="$4"
  local HOST="$5"
  shift 5

  cat << TPL
#--
# Init the first node.  This creates the schema '_$CLUSTER'
# containing all replication system specific database objects.
#--
echo 'Init cluster $ID - $CLUSTER : $NODEID - $DB - $HOST';
try{
  init cluster (ID=$ID,
				COMMENT = '$CLUSTER : $NODEID - $DB - $HOST');
}
TPL
}

function tpl_slonik_try_drop_path {
  local SERVER1="$1"
  local SERVER2="$2"
  shift 2

  cat << TPL
echo 'Drop path $SERVER1 <-> $SERVER2';
try{
  drop path (server = $SERVER1, client = $SERVER2, event node = $SERVER1 );
  drop path (server = $SERVER2, client = $SERVER1, event node = $SERVER2 );
}
TPL
}

function tpl_slonik_try_execute_script {
  local ID="$1"
  local FILENAME="$2"
  local EVENT_NODE="$3"
  shift 3

  cat << TPL
echo 'Propagation script : ' ;
try{
  EXECUTE SCRIPT (
  SET ID = $ID,
  FILENAME = '$FILENAME',
  EVENT NODE = $EVENT_NODE );
}
TPL
}

function tpl_slonik_try_failover {
  local ID="$1"
  local BACKUP_NODE="$2"
  shift 2

  cat << TPL
echo 'Failover $ID - $BACKUP_NODE ';
try{
  failover (ID = $ID,
			BACKUP NODE = $BACKUP_NODE
			);
}
TPL
}

function tpl_slonik_try_unlock_set {
  local ID="$1"
  local ORIGIN="$2"
  shift 2

  cat << TPL
try{
  echo 'unlock $ID : ';
  unlock set (ID = $ID,
			  ORIGIN = $ORIGIN );
}
TPL
}

function tpl_slonik_try_move_set {
  local ID="$1"
  local OLDORIGIN="$2"
  local NEWORIGIN="$3"
  shift 3

  cat << TPL
try{
  echo 'move set $ID : ';
  lock set (ID = $ID,
			ORIGIN = $OLDORIGIN ) ;
  echo 'lock set : OK';
  echo 'move set : ';
  move set (ID = $ID,
			OLD ORIGIN = $OLDORIGIN,
			NEW ORIGIN = $NEWORIGIN );
}
TPL
}

#
# Without TRY
#
function tpl_slonik_set_add_table {
  local SET_ID="$1"
  local ORIGIN="$2"
  local ID="$3"
  local NAME="$4"
  shift 4

  cat << TPL
  set add table (SET ID=$SET_ID, ORIGIN=$ORIGIN, ID=$ID, FULLY QUALIFIED NAME = '$NAME', COMMENT='table $NAME');
TPL
}

function tpl_slonik_set_drop_table {
  local ORIGIN="$1"
  local ID="$2"
  shift 2
  cat << TPL
  set drop table (ORIGIN = $ORIGIN,
				  ID = $ID);
TPL
}

function tpl_slonik_set_add_sequence {
  local SET_ID="$1"
  local ORIGIN="$2"
  local ID="$3"
  local NAME="$4"
  shift 4

  cat << TPL
  set add sequence (SET ID=$SET_ID, ORIGIN=$ORIGIN, ID=$ID, FULLY QUALIFIED NAME = '$NAME', COMMENT='sequence $NAME');
TPL
}

function tpl_slonik_set_drop_sequence {
  local ORIGIN="$1"
  local ID="$2"
  shift 2
  cat << TPL
  set drop sequence (ORIGIN = $ORIGIN,
					ID = $ID);
TPL
}

function tpl_slonik_store_path {
  # use global SLONY_PWD if defined and SLONY_USER
  local SERVER="$1"
  local CLIENT="$2"
  local DB="$3"
  local HOST="$4"
  local PORT="$5"
  shift 5

  cat << TPL
  echo 'Store path between server $SERVER and client $CLIENT';
  store path (SERVER = $SERVER,
			  CLIENT = $CLIENT,
			  CONNINFO='dbname=$DB host=$HOST port=$PORT user=${SLONY_USER}${SLONY_PWD:+ password=$SLONY_PWD}');
TPL
}

function tpl_slonik_update_functions {
  local ID="$1"
  shift

  cat << TPL
  echo 'Node $ID:';
  update functions ( ID=$ID );
TPL
}

function tpl_slonik_create_set {
  local ID="$1"
  local ORIGIN="$2"
  local HOST="$3"
  shift 3

  cat << TPL
  echo 'Create set $ID';
  create set (ID=$ID, ORIGIN=$ORIGIN, COMMENT='Tables in ${HOST}');
TPL
}

# End tpl slonik functions
#################################################

#################################################
# Begin tpl bash functions
function tpl_sh_shabang {
  echo "#!/bin/bash"
}
function tpl_sh_slonik_begin_eof {
  echo "${SLON_BIN}/slonik << SEOF"
}
function tpl_sh_slonik_end_eof {
  echo "SEOF"
}

function tpl_sh_kill_slon_daemons {
  local CLUSTER="$1" ; shift

  cat << TPL
echo "Stopping $CLUSTER replication..."
for processus in \`ps auxw | grep "\bslon\b" | grep "\b$CLUSTER\b" | grep -v sh | awk '{print \$2}'\`
do
  kill -15 \${processus}
done
echo "Slon daemons stopped on \`hostname\`"
TPL
}

function tpl_sh_start_slon_daemons {
  # use global SLON_ETC SLON_BIN SLONY_USER and psql_exec
  local CLUSTER="$1"
  local NODE="$2"
  local DB="$3"
  local HOST="$4"
  local PORT="$5"
  shift 5

  cat << TPL
${SLON_BIN}/slon -f $SLON_ETC/slon.cfg ${CLUSTER} "dbname=$DB host=$HOST port=$PORT user=${SLONY_USER}${SLONY_PWD:+ password=$SLONY_PWD}" >${SLON_LOG}/${CLUSTER}_${DB}_${NODE}.log 2>&1 &
TPL
}

function tpl_sh_require_plpgsql {
  # use global SLONY_USER and psql_exec
  local DB="$1"
  local HOST="$2"
  local PORT="$3"
  shift 3

  cat << TPL
# Do we need to add PL/pgSQL ?
# Yes, so do it.
CREATE_PLPGSQL=\`${psql_exec} ${DB} -t -U ${SLONY_USER} -h ${HOST} -p ${PORT} -c "select 1 from pg_language where lanname = 'plpgsql';"\`
if [ ! \$CREATE_PLPGSQL ] || [ \$CREATE_PLPGSQL -ne 1 ] ; then
  $PG/bin/createlang plpgsql -U ${SLONY_USER} ${DB} -h ${HOST} -p ${PORT}
  if [ \$? != 0 ] ; then
	 echo "Fail to create plpgsql, aborting"
	exit 1
  fi
  echo "plpgsql created on ${DB} - ${HOST}"
fi
TPL
  return 0
}

function tpl_sh_uninstall_slon_schema {
  # use global SLONY_USER and psql_exec
  local DB="$1"
  local HOST="$2"
  local PORT="$3"
  local CLUSTER="$4"
  shift 4

  cat << TPL
# We uninstall slony cleanly (put objects in their original state)
# Then we drop slon specific schema
echo "Uninstalling node and dropping schema on ${DB} - ${HOST}..."
PURGE_SLON=\`${psql_exec} ${DB} -t -U ${SLONY_USER} -h ${HOST} -p ${PORT} -c "select 1 from pg_namespace where nspname = '_${CLUSTER}';"\`
if [ \$PURGE_SLON ] && [ \$PURGE_SLON -eq 1 ] ; then
  ${psql_exec} ${DB} -U ${SLONY_USER} -h ${HOST} -p ${PORT} -c 'select "_${CLUSTER}".uninstallnode( );'
  ${psql_exec} ${DB} -U ${SLONY_USER} -h ${HOST} -p ${PORT} -c 'drop schema "_${CLUSTER}" cascade;'
  if [ \$? != 0 ] ; then
	echo "Fail to drop existent schema _${CLUSTER}, aborting"
	exit 1
  fi
  echo "Schema _${CLUSTER} removed from ${DB} - ${HOST}"
fi
TPL
  return 0
}

function tpl_sh_while_subscribe {
  # use global SLONY_USER and psql_exec
  local SLAVE="$1"
  local MASTER="$2"
  local TABLE="$3"
  local HOST="$4"
  local PORT="$5"
  local CLUSTER="$6"
  local SUB_RECEIVER="$7"
  local SUB_SET="$8"
  shift 4

  cat << TPL
while [ 1 ] ; do
  echo -n "Waiting ${SLAVE} has subscribed to ${MASTER}"
  if [ x\`$PG/bin/psql -t ${TABLE} -U ${SLONY_USER} -h ${HOST} -p ${PORT} -c "select sub_active from _$CLUSTER.sl_subscribe where sub_receiver=${SUB_RECEIVER} and sub_set=$SUB_SET" |awk '{print \\\$1}' |head -n 1\` = "xt" ]; then
	echo "OK"
	break;
  else
	echo -n "."
	sleep 5
  fi
done
TPL
  return 0
}

# End tpl bash functions
#################################################
