#!/bin/bash
# Auteur : Stéphane Schildknecht
# 2006-11-17
# 2007/03/20 : Prise en compte des réplications en cascade
# Modifications :
#  - 20071214 Bernard Choppy
#    possibilite de lancer depuis n'importe ou
#    systematisation de l'utilisation des variables
#

# create_slon.sh
# Setting up a replication

#################################################
# Begin initialise configuration
CDIR=`dirname $0`
pushd $CDIR/.. >> /dev/null
SLON=`pwd`
popd >> /dev/null

. $SLON/etc/slony_include.h
. $SLON_ETC/fonctions.sh
. $SLON_ETC/tpl_functions.sh

filename="$0"

export LD_LIBRARY_PATH=$PG/lib/
# End initialise configuration
#################################################

#################################################
# Begin help, getopt and handle case
slhelp() {
  echo $(eval_gettext "\$filename")
  echo $(gettext      "Creation of scripts to initialize a replication")
  echo ""
  echo $(eval_gettext "Usage : \$filename *options*")
  echo $(gettext      "Options :")
  echo -e $(gettext   "\\t-h, --help\\t\\t\\tPrint this help")
  echo -e $(gettext   "\\t-c, --cluster <instance>\\tThe replication instance name")
  echo -e $(gettext   "\\t-s, --set <set>\\t\\t\\tThe number of the set to create. When not specified, the whole replication is initialised.")
  echo ""
}

TEMP=`getopt -n $0 -o h,c:s: -l help,cluster:,set: -- "$@"`
if [ $? != 0 ] ; then
  slhelp
  exit 1
fi
eval set -- "$TEMP"

INSTANCE=""
SET=0
while [ $# -ge 1 ]; do
  case "$1" in
	-h|--help)
	  HELP=1
	  slhelp
	  exit 0
	  ;;
	-c|--cluster)
	  shift
	  INSTANCE="$1"
	  ;;
	-s|--set)
	  shift
	  SET="$1"
	  ;;
	--)
	  shift
	  break
	  ;;
	*)
	  echo $(gettext "Warning !")
	  slhelp
	  exit 1
	  ;;
  esac
  shift
done

if [ -z $INSTANCE ]; then
  slhelp;
  exit 1
fi
# End help, getopt and handle case
#################################################

#################################################
# Begin Sanity checks
relbase_exist
replication_exist $INSTANCE
# End Sanity checks
#################################################

#################################################
# Begin Real work

# TODO what is the good name ?
CLUSTERNAME=$INSTANCE

###############
## FR : On fonctionne en deux temps :
# 1. Les requêtes systèmes permettant d'ajouter les langages procéduraux, de
# supprimer les schémas éventuellement existant et d'arrêter les daemons
# slon s'il en existe
# 2. le script slonik d'initialisation de la réplication
###############
# On lit toutes les lignes du fichier.
# Pour toutes celles qui correspondent à l'instance,
# S'il s'agit de la première (maître),
# Alors, on initie la réplication (init cluster)
# Sinon, on ajoute le noeud (store node)
########################################
## EN: We act in two times
# 1. We add procedural language, drop any existing _CLUSTER schema,
# stop any existing slon daemon dealing with concerned replication
# 2. We initiate the replication with a slonik script
##########
# The master of the replication is arbitrarily defined.
# We take the first node of bases.h file.
# For that node, we issue INIT CLUSTER.
# For every other node, we do STORE NODE.
##########

echo $(eval_gettext "Creation of initialisation file for replication of \$CLUSTERNAME databases.") >&2

#
# Node information array creation
#
node $CLUSTERNAME ;

#
# Arbritaly define the first node in bases.h as our master, NODE[1]
#
MASTERNODE=${NODE[1]} # On ne peut pas faire plus arbitraire :-(
MASTERHOST=${NODE[${INDEX[$MASTERNODE]}+2]}
MASTERPORT=${NODE[${INDEX[$MASTERNODE]}+3]}

#################################################
# Begin Creation of shell script

#
# Construct the bash and slonik script to start replication
#
tpl_sh_shabang
echo "# Initialising replication"

#
# If no SET specified, replication is initialized
#
if [ $SET -eq 0 ]; then
  tpl_sh_kill_slon_daemons "$CLUSTERNAME"

  # For each nodes/db involved in the replication
  for (( i=1 ; i <= taille_nod ; i+=4 )); do
	echo "###"
	echo "#${NODE[$i+1]} - ${NODE[$i+2]}"

	# Adding pl/pgsql if required
	tpl_sh_require_plpgsql \
		"${NODE[$i+1]}" "${NODE[$i+2]}" "${NODE[$i+3]}"

	# Dropping slony schema, if any
	tpl_sh_uninstall_slon_schema \
		"${NODE[$i+1]}" "${NODE[$i+2]}" "${NODE[$i+3]}" \
		"$CLUSTERNAME"

	echo "###"
  done
fi

echo "# Slonik script for initialising replication"
tpl_sh_slonik_begin_eof

tpl_slonik_include_preamble "${SLON_ETC}/$INSTANCE.preamble"

# Only one init by replication, only one store node by node
# i.e., if $SET specified, this part is not done.
if [ $SET -eq 0 ]; then
  tpl_slonik_init_cluster \
	  $MASTERNODE $INSTANCE \
	  ${NODE[1]} ${NODE[2]} ${NODE[3]}
  tpl_slonik_success_error \
	  "$(eval_gettext 'Cluster $MASTERNODE : Success')" \
	  "$(eval_gettext 'Cluster $MASTERNODE : Failled!!!')"

  echo "# The other nodes involved in the replication"

  for (( i=5 ; i <= taille_nod ; i+=4 )); do
	SLAVENODE=${NODE[$i]}
	SLAVEBASE=${NODE[$i+1]}
	SLAVEHOST=${NODE[$i+2]}
	SLAVEPORT=${NODE[$i+3]}

	tpl_slonik_store_node \
		"$SLAVENODE" "$MASTERNODE" "$SLAVEBASE" "$SLAVEHOST" \
		"$CLUSTERNAME" "Store node $SLAVENODE"
	tpl_slonik_success_error \
		"$(eval_gettext 'Store node $SLAVENODE : Success')" \
		"$(eval_gettext 'Store node $SLAVENODE : Failed !!!')"
  done
  for (( i=1 ; i <= taille_nod ; i+=4 )); do
	SLAVENODE=${NODE[$i]}
	SLAVEBASE=${NODE[$i+1]}
	SLAVEHOST=${NODE[$i+2]}
	SLAVEPORT=${NODE[$i+3]}

	#Liens entre un noeud et les autres (numéros inférieurs)
	# How each node is linked to the others
	for (( j=1 ; j <= taille_nod ; j+=4 )); do
	  if [ $i -ne $j ]; then
		NODE2=${NODE[$j]}
		BASE2=${NODE[$j+1]}
		HOST2=${NODE[$j+2]}
		PORT2=${NODE[$j+3]}

		tpl_slonik_store_path \
			"$SLAVENODE" "$NODE2" \
			"$SLAVEBASE" "$SLAVEHOST" "$SLAVEPORT"
	  fi
	done
  done

  GREP_SET="grep \"^$INSTANCE\b\" $SLON_ETC/relations.h | grep -v \"^#\" | awk '{print \$2}' | sort -u"
else
  GREP_SET="echo $SET"
fi


# For every set in the replication
for SETID in `eval $GREP_SET` ; do
  relation $SETID

  cat <<EOF

#--
# Slony-I organizes tables into sets.  The smallest unit a node can
# subscribe is a set. The master or origin of the set is node $MASTER.
# Les tables sont organisées au sein de set, plus petite unité de réplication.
# L'origine du set est $MASTER.
#--
try{
EOF

  tpl_slonik_create_set $SETID $MASTER ${NODE[2]}
  echo "  # Tables"

  if [ $SET -eq 0 ]; then
	TABLEID=1
  else
	# Récupération du dernier numéro utilisé dans la numérotation des tables
	# We get the last used tab_id.
	# TODO make a function of this and solve the other TODO about max tab_id
	TABLEID=`$PG/bin/psql ${NODE[2]} -U ${SLONY_USER} -p${NODE[4]} -h${NODE[3]} -t -c "select max(tab_id) +1 from \"_${INSTANCE}\".sl_table ;"| awk '{print $1}'`
  fi

  for table in `$PG/bin/psql -t -h $MASTERHOST -p$MASTERPORT ${NODE[2]} -U ${SLONY_USER} -c "select schemaname||'.'||tablename from pg_tables where schemaname= 'public' order by tablename asc;"` ; do
	  tpl_slonik_set_add_table "$SETID" "$MASTER" "$TABLEID" "$table"
	  TABLEID=$(($TABLEID +1))
  done

  echo "# Sequences"

  if [ $SET -eq 0 ]; then
	SEQID=1
  else
	# Récupération du dernier numéro utilisé dans la numérotation des séquences
	# We get the last used seq_id
	# TODO make a function of this and solve the other TODO about max seq_id
	SEQID=`$PG/bin/psql ${NODE[2]} -U ${SLONY_USER} -p${NODE[4]} -h${NODE[3]} -t -c "select max(seq_id) +1 from \"_${INSTANCE}\".sl_sequence;"| awk '{print $1}'`
  fi

  for SEQ in `$PG/bin/psql -t -h $MASTERHOST -p $MASTERPORT ${NODE[2]} -U ${SLONY_USER} -c "select ns.nspname||'.'||cl.relname from pg_class cl inner join pg_namespace ns on ns.oid=cl.relnamespace where nspname='public' and relkind='S' order by cl.relname asc;"` ; do
	tpl_slonik_set_add_sequence "$SETID" "$MASTER" "$SEQID" "$SEQ"
	SEQID=$(($SEQID +1))
  done

  echo "}"
  tpl_slonik_success_error \
	  "$(gettext 'Creation Set : Success')" \
	  "$(gettext 'Creation Set : Failed !!!')"
done


echo "exit 0;"
tpl_sh_slonik_end_eof
cat << EOF
if [ \$? -ne 0 ]; then
  echo "Error slonik while creating set"
  exit 1
fi

# We start replication. All daemons on the same machine, which is
# the one on which that script is executed.
EOF
for (( i=1 ; i <= taille_nod ; i+=4 )); do
  cat << EOF
${SLON_BIN}/slon -f $SLON_ETC/slon.cfg ${INSTANCE} "dbname=${NODE[$i+1]} host=${NODE[$i+2]} port=${NODE[$i+3]} user=${SLONY_USER}${SLONY_PWD:+ password=$SLONY_PWD}" >${SLON_LOG}/${CLUSTERNAME}_${NODE[$i+1]}_${NODE[$i]}.log 2>&1 &

echo "Slon daemon for ${NODE[$i+1]} - ${NODE[$i+2]} started on `hostname`"
EOF
done

cat <<EOF

# Il faut maintenant informer les bases de la réplication
## EN: And now, subscription
EOF

  #Pour tous les sets de réplication
  # For every set
for SETID in `eval $GREP_SET` ; do
  relation $SETID
  for (( i=1 ; i <= taille_rel ; i+=2 )); do
	MASTERNODE=${REL[$i]}
	SLAVENODE=${REL[$i+1]}
	mtr=${INDEX[${REL[$i]}]}
	slv=${INDEX[${REL[$i+1]}]}

	echo "echo '${SLAVENODE} subscribes to Set ${SETID}'"

	if [ $MASTERNODE -ne $MASTER ]; then
	  tpl_sh_while_subscribe $MASTERNODE $MASTER ${NODE[$mtr+1]} \
							 ${NODE[$mtr+2]} ${NODE[$mtr+3]} \
							 $INSTANCE ${NODE[$mtr]} $SETID
	fi

	#
	# Call slonik to subscribe set/node
	#
	tpl_sh_slonik_begin_eof
	tpl_slonik_include_preamble "${SLON_ETC}/$INSTANCE.preamble"
	tpl_slonik_subscribe_set \
		$SETID $MASTERNODE $SLAVENODE \
		"Node $SLAVENODE subscribes set $SETID"
	tpl_slonik_success_error \
		"Subscription Successfull" \
		"Subscription Failled !!!"
	tpl_sh_slonik_end_eof
  done
done

# TODO shouldn't this be in the loop ?
cat <<EOF
if [ \$? -ne 0 ]; then
  echo "Error slonik while subscribing set"
  exit 1
fi
exit 0

EOF
# End Creation of shell script
#################################################

exit 0
# End Real work
#################################################
