#!/bin/bash
# Auteur : Stephane Schildknecht
# 2007-01-15
# REV 2007-05-14 : Prise en compte de la nouvelle architecture
# Modifications :
#  - 20071214 Bernard Choppy
#    possibilite de lancer depuis n'importe ou
#    systematisation de l'utilisation des variables
#  - 20080401 SAS
#    Le subscribe est effectué avant le move set
#    Cela permet de supprimer les souscripteurs directs d'un noeud a supprimer
#

# slony_switch.sh
## FR : Script de bascule contrôlée entre deux noeuds
## EN: Switchover script between 2 nodes

#################################################
# Begin initialise configuration
CDIR=`dirname $0`
pushd $CDIR/.. >> /dev/null
SLON=`pwd`
popd >> /dev/null

. $SLON/etc/slony_include.h
. $SLON_ETC/fonctions.sh
. $SLON_ETC/tpl_functions.sh

filename="$0"

export LD_LIBRARY_PATH=$PG/lib/
# End initialise configuration
#################################################

#################################################
# Begin help, getopt and handle case
slhelp() {
  echo $(eval_gettext "\$filename")
  echo ""
  echo $(eval_gettext "Usage: \$filename *options*")
  echo -e $(gettext "\\tSwitchover")
  echo -e $(gettext "\\tOptions:")
  echo -e $(gettext "\\t\\t-h, --help\\t\\tPrint this help")
  echo -e $(gettext "\\t\\t-c, --cluster\\t\\tThe replication cluster name")
  echo -e $(gettext "\\t\\t-f, --failed\\t\\tThe failed node (ancient provider)")
  echo -e $(gettext "\\t\\t-b, --backup\\t\\tThe backup node (new provider)")
  echo ""
}

TEMP=`getopt -n $0 -o c:f:b:,h -l help,cluster:,failed:,backup -- "$@"`
if [ $? != 0 ] ; then
  slhelp
  exit 1
fi
eval set -- "$TEMP"

INSTANCE=""
FAILED=""
BACKUP=""
while [ $# -ge 1 ]; do
  case "$1" in
	--help)
	  HELP=1
	  slhelp
	  exit 0
	  ;;
	-c|--cluster)
	  shift
	  INSTANCE="$1"
	  ;;
	-f|--failed)
	  shift
	  FAILED="$1"
	  ;;
	-b|--backup)
	  shift
	  BACKUP="$1"
	  ;;
	--)
	  shift
	  break
	  ;;
	*)
	  echo $(gettext "Warning !")
	  slhelp
	  exit 1
	  ;;
  esac
  shift
done

if [ x"$INSTANCE" = "x" -o x"$FAILED" = "x" -o x"$BACKUP" = "x" ]; then
  slhelp
  exit 1
fi

# End help, getopt and handle case
#################################################

#################################################
# Begin Sanity checks
relbase_exist
preamble_exist $INSTANCE
replication_exist $INSTANCE
# End Sanity checks
#################################################

#################################################
# Begin Real work

MASTERNODE=$FAILED
SLAVENODE=$BACKUP

## FR : Afin d'automatiser la bascule, il est nécessaire de récupérer
# les informations concernant le maitre et l'esclave.
# Dans ce cas, le fichier de préambule doit être à jour.
# Si ce n'était pas le cas, peut-être faut-il le recréer.
# Cela n'est pas fait dans ce script.

# Il est impératif que les fichiers ${SLON_ETC}/bases.h et
# ${SLON_ETC}/$CLUSTERNAME.preamble soient à jour.

## EN: To automate the switch, we need acurate information on nodes
# (master-slave). Preamble file has to be up to date. We can't test that here.
# Its re-creation should be thought about out of that script.
# Same for ${SLON_ETC}/bases.h and ${SLON_ETC}/relations.h. We test if they exist.

## FR : Création des tableaux
## EN: Creation of arrays
node $INSTANCE

MASTERDB=${NODE[${INDEX[$MASTERNODE]}+1]}
MASTERHOST=${NODE[${INDEX[$MASTERNODE]}+2]}
MASTERPORT=${NODE[${INDEX[$MASTERNODE]}+3]}

## FR : Mise à jour du fichier de relations
## EN: We try to recreate relations file
relation_file $INSTANCE $MASTERDB $MASTERHOST $MASTERPORT

SUPER=0
GREP_SET="grep \"^$INSTANCE\b\" $SLON_ETC/relations.h | grep -v \"^#\" | awk '{print \$2}' | sort -u"

for SETID in `eval $GREP_SET` ; do
  relation $SETID
  echo "set : $SETID, MASTER=$MASTER"

  if [ $MASTERNODE -eq $MASTER ]; then
## FR : Pour tous les autres souscripteurs, on resouscrit au nouveau maître de réplication
## EN: Every other subscriber is subscribed to new provider
	for (( j=1 ; j <= taille_rel ; j+=2 )); do
	  if [ ${REL[$j]} -eq $MASTERNODE ]; then
		OLDSLAVE=${REL[$j+1]}
		let mtr=${INDEX[${REL[$j]}]}
		let slv=${INDEX[${REL[$j+1]}]}
		if [ $OLDSLAVE -ne $SLAVENODE ]; then
		  echo MASTER : $SLAVENODE \| NODE : $OLDSLAVE
		  ${SLON_BIN}/slonik <<_EOF_
`tpl_slonik_include_preamble "${SLON_ETC}/$CLUSTER.preamble"`
`tpl_slonik_subscribe_set $SETID $SLAVENODE $OLDSLAVE \
						 "$OLDSLAVE subscribes SET $SETID on $SLAVENODE:"`
on success { echo 'OK' ;}
on error { echo 'KO' ;}
_EOF_
		fi
	  fi
	done

## FR : On switche le maître
## EN: master switch
	${SLON_BIN}/slonik <<_EOF_
`tpl_slonik_include_preamble "${SLON_ETC}/$CLUSTER.preamble"`

`tpl_slonik_try_unlock_set $SETID $MASTERNODE`
on success { echo 'OK' ;}
on error { echo 'KO' ;}

`tpl_slonik_try_move_set $SETID $MASTERNODE $SLAVENODE`
on error { echo 'KO' ;}
on success { echo 'OK' ;}
_EOF_

	else
	## FR : Pour tous les esclaves
	# Peut-on vérifier qu'un noeud ne souscrit pas à un set donné ?
	## EN: For every slave (How should we verify a node does not subscribe
	# to a given set?
	  for (( j=1 ; j <= taille_rel ; j+=2 )); do
		if [ ${REL[$j]} -eq $MASTERNODE ]; then
		  OLDSLAVE=${REL[$j+1]}
		  if [ $OLDSLAVE -ne $SLAVENODE ]; then
			${SLON_BIN}/slonik <<_EOF_
			`tpl_slonik_include_preamble "${SLON_ETC}/$INSTANCE.preamble"`
			# Slave subscribes to provider of switched node
			`tpl_slonik_subscribe_set $SETID $MASTER $OLDSLAVE "$OLDSLAVE subscribes $SETID on $MASTER: "`
# 			tpl_slonik_success_error "$(gettext 'Success !!!')" \
# 									 "$(gettext 'Failed')"

			on success { echo 'OK' ;}
			on error { echo 'KO' ;}
			# Node subscribes ancient slave
			`tpl_slonik_subscribe_set $SETID $OLDSLAVE $MASTERNODE "Souscription set $SETID par $MASTERNODE : "`

			on success { echo 'OK' ;}
			on error { echo 'KO' ;}

_EOF_
		  fi
		fi
	  done
	fi
  done

MASTERDB=${NODE[${INDEX[$SLAVENODE]}+1]}
MASTERHOST=${NODE[${INDEX[$SLAVENODE]}+2]}
MASTERPORT=${NODE[${INDEX[$SLAVENODE]}+3]}

## FR : Il est nécessaire d'attendre un peu...
## EN: How could sleep long enough, but not too much?
sleep 30s

## FR : Mise à jour du fichier de relations
## EN: Updating relations.h file
relation_file $INSTANCE $MASTERDB $MASTERHOST $MASTERPORT

## FR : Il est nécessaire de trier le ficher de relations...
# Dans l'immédiat, ce tri doit être fait à la main.
## EN: Relation file should be sorted. Today sort has to be done manually.

exit 0
# End Real work
#################################################
